--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Gui = P:NewModule("Gui", "AceEvent-3.0")

local ST = LibStub("ScrollingTable")

local bgcolor_
local colors_
local cur_color_ = 0
do 
  local dark_a = { r = 0.2, g = 0.2, b = 0.2, a = 0.5 }
  local dark_b = { r = 0.25, g = 0.25, b = 0.25, a = 0.5 }
  bgcolor_ = { dark_a, dark_b, dark_a, dark_b, dark_a, dark_b, dark_a, dark_b }
  local light_a = { r = 0.95, g = 0.95, b = 0.95, a = 1 }
  local light_b = { r = 1, g = 1, b = 1, a = 1 }
  local med_a = { r = 0.8, g = 0.8, b = 0.8, a = .85 }
  local med_b = { r = 0.85, g = 0.85, b = 0.85, a = .85 }
  colors_ = {
    [0] = { light_a, light_b, light_a, light_b, light_a, light_b, light_a, light_b },
    [1] = { med_a, med_b, med_a, med_b, med_a, med_b, med_a, med_b, },
  }
end

local function TableSum(res, a)
  for k, v in pairs(a) do res[k] = (res[k] or 0) + v end
end

local function ShallowTableEqual(a, b)
  if not a or not b then return false end 
  for k, v in pairs(a) do if b[k] ~= v then return false end end
  for k, v in pairs(b) do if a[k] ~= v then return false end end
  return true
end

local num_rows_ = 0
local function GetRow_(rows, i)
  local row = rows[i]
  if not row then
    row = {}
    rows[i] = row
    tinsert(P.gui_.data, row)
    num_rows_ = num_rows_ + 1
  end
  return row, i + 1
end

local function CountF_(tbl, item)
  return {
    value = function()
      local count = tbl[item]
      return count and count > 0 and count or ''
    end,
  }
end

local function CostBasisF_(item)
  return {
    value = function()
      return P.moneyString(P:QueryItem(item))
    end
  }
end

-- Columns:Item, Count, Expected, CB, Source, Count, CB, Cost
local last_expected_, last_from_, last_row_, from_sum_, to_sum_, expect_sum_, cost_sum_
function P:LogTransactionToGui(gold_cost, from_items, to_items, to_expected)
--  if not P.G.experimental_gui then return end
  gold_cost = gold_cost or 0
  from_items = from_items or {}
  to_items = to_items or {}
  to_expected = to_expected or to_items
  local new = false
  if not ShallowTableEqual(to_expected, last_expected_) or not ShallowTableEqual(from_items, last_from_) then
    last_expected_ = to_expected
    last_from_ = from_items
    from_sum_ = {}
    to_sum_ = {}
    expect_sum_ = {}
    cost_sum_ = { 0 }
    new = true
  end
  TableSum(from_sum_, from_items)
  TableSum(to_sum_, to_items)
  TableSum(expect_sum_, to_expected)
  cost_sum_[1] = cost_sum_[1] + gold_cost
  
  if new then
    local rows = {}
    local row = GetRow_(rows, 1)
    local cost_tbl = cost_sum_
    row[8] = {
      value = function()
        if cost_tbl[1] > 0 then return P.moneyString(cost_tbl[1]) end
        if cost_tbl[1] < 0 then return '-' .. P.moneyString(cost_tbl[1]) end
      end
    }
    
    local i = 1
    for item in pairs(expect_sum_) do
      row, i = GetRow_(rows, i)      
      row[1] = { value = self.itemIconLink(item) .. self.linkOfItem(item) }
      row[2] = CountF_(to_sum_, item)
      row[3] = CountF_(expect_sum_, item)
      row[4] = CostBasisF_(item)
    end
    
    i = 1
    for item in pairs(from_sum_) do
      row, i = GetRow_(rows, i)
      row[5] = { value = self.itemIconLink(item) .. self.linkOfItem(item) }
      row[6] = CountF_(from_sum_, item)
      row[7] = CostBasisF_(item)
    end
    
    local color_only = { color = colors_[cur_color_][i] }
    for _, row in ipairs(rows) do
      for i = 1, 8 do
        if not row[i] then
          row[i] = color_only
        else
          row[i].color = colors_[cur_color_][i]
        end
      end
    end
    cur_color_ = 1 - cur_color_
  end

  -- TODO: fix this so it scrolls to the bottom
  local scroll_to = max((num_rows_ - 24) * 15 - 3, 0)
  self.gui_.table.scrollframe:SetVerticalScroll(scroll_to)
  self.gui_.table:SortData()
--  self.gui_.table:Refresh()
--[[
  P:DebugPrint({
    range = self.gui_.table.scrollframe:GetVerticalScrollRange(),
    offset = FauxScrollFrame_GetOffset(self.gui_.table.scrollframe),
    scroll_to = scroll_to,
  })
--]]
  if P.G.gui == "fade" then
    self.gui_.anim:Stop()
    self.gui_.anim:Play()
  end
end

local function NullCmp_(table, rowa, rowb, column)
  return rowa < rowb
end

function Gui:OnInitialize()
  local frame = CreateFrame("Frame", "CostBasisUIFrame", UIParent)
  frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
  frame:SetSize(833, 400)  -- Width is 33 plus the sum of column widths.
  frame:EnableMouse()
  frame:SetMovable(true)
  frame:SetFrameStrata("MEDIUM")
  frame:SetToplevel(true)
  local button = CreateFrame("Button", nil, frame)
  button:SetPoint("TOPLEFT", 0, 0)
  button:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, -24)
  button:SetSize(1000, 1000)
  button:RegisterForDrag("LeftButton")
  button:SetScript("OnDragStart", function() frame:StartMoving() end)
  button:SetScript("OnDragStop", function() frame:StopMovingOrSizing() end)
  local buttonbg = button:CreateTexture(nil, "BACKGROUND")
  buttonbg:SetTexture([[Interface\PaperDollInfoFrame\UI-GearManager-Title-Background]])
  buttonbg:SetPoint("TOPLEFT")
  buttonbg:SetPoint("BOTTOMRIGHT")
  local close = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
  close:SetPoint("TOPRIGHT", 0, 0)
  close:SetSize(24, 24)
  button:SetScript("OnClick", function() frame:Hide() end)
  local tbl = ST:CreateST({
    { name = "Item", width = 150, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[1] },
    { name = "Count", width = 60, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[2] },
    { name = "Expected", width = 80, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[3] },
    { name = "CB", width = 100, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[4] },
    { name = "Source", width = 150, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[5] },
    { name = "Count", width = 60, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[6] },
    { name = "CB", width = 100, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[7] },
    { name = "Cost", width = 100, align = "CENTER", comparesort = NullCmp_, bgcolor=bgcolor_[8] },
  }, 24, nil, nil, frame)
  tbl.frame:SetPoint("TOPLEFT", 0, -20)
  for i = 1, 8 do
    tbl.head.cols[i]:EnableMouse(false)
  end
  local data = {}
  tbl:SetData(data, true)
  if P.G.gui == "show" then
    frame:Show()
  else
    frame:Hide()
  end
  
  local group = frame:CreateAnimationGroup()
  local anim = group:CreateAnimation("Alpha")
  anim:SetDuration(1)
  anim:SetStartDelay(5)
  anim:SetChange(-1)
  anim:SetScript("OnFinished", function() frame:Hide() end)
  anim:SetScript("OnPlay", function() frame:SetAlpha(1.0); frame:Show() end)
  
  P.gui_ = { frame = frame, table = tbl, data = data, anim = group }
  
  P:RegisterSlashVerb("log", nil, "shows transaction log", function()
    P.gui_.anim:Stop()
    P.gui_.frame:SetAlpha(1)
    P.gui_.frame:Show()
  end)
end
