--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Trade = P:NewModule("Trade", "AceEvent-3.0")

local ItemSet = P.ItemSet
local Timer = P.Timer

local fromItems, toItems, cost, timer, inTimer

-- on success, TRADE_CLOSED is fired twice
-- on cancel, TRADE_CLOSED is fired twice and TRADE_REQUEST_CANCEL once (in some order)

local function onUpdate(event,...)
	P:DebugPrint({event=event,args={...}})
	cost = GetPlayerTradeMoney() - GetTargetTradeMoney()
	fromItems = ItemSet.new()
	toItems = ItemSet.new()
	for i = 1,MAX_TRADE_ITEMS do
		local _,_,count = GetTradePlayerItemInfo(i)
		local item = GetTradePlayerItemLink(i)
		if item then fromItems:Inc(P.idOfItem(item),count) end
		_,_,count = GetTradeTargetItemInfo(i)
    item = GetTradeTargetItemLink(i)
		if item then toItems:Inc(P.idOfItem(item),count) end
	end
end

local function doClear()
	fromItems = nil
	toItems = nil
	cost = nil
	inTimer = nil
	if timer then timer:Cancel() end
end

--[[
local function doTrade()
	P:DebugPrint({fromItems=fromItems,toItems=toItems,cost=cost})
	if fromItems and toItems and cost then
		local item,count = next(toItems)
		local function record()
			CB:RecordTransaction(cost,fromItems,toItems)			
			doClear()
		end
		if item then -- if we're getting at least 1 item, then make sure that the trade actually happens (not inventory full!)
			P:ReceiveCallback(item,count,record,function() P:DebugPrint("receive failed"); doClear() end)
		else
			record()
		end
	end
end
]]

local function onUpdateAccept(event,player,target,...) -- 0/1 values: 1=accepted, 0=not accepted
	local doTrade = (player==1) and (target==1)
	P:DebugPrint({
		player=player,
		target=target,
		fromItems=fromItems,
		toItems=toItems,
		cost=cost,
		doTrade=doTrade,
		args={...},
	})
end

local function record()
	CB:RecordTransaction(cost,fromItems,toItems)			
	doClear()
end

local function onTimer()
	if fromItems and toItems and cost then
		record()
	else
		doClear()
	end
end
	

local function onClosed(event,...)
	P:DebugPrint({event=event,args={...}})
	if fromItems and toItems and cost and not inTimer then
		inTimer=true
		local item,count = next(toItems)
		if item then -- if we're getting at least 1 item, then make sure that the trade actually happens (not inventory full!)
			P:ReceiveCallback(item,count,record,function() P:DebugPrint("receive failed"); doClear() end)
		else
			timer=Timer.updateOrNew(timer,onTimer,5*P.EPSILON_TIME)
		end
	end
end

local function onCancel(event,...)
	P:DebugPrint({event=event,args={...}})
	doClear()
end

function Trade:OnInitialize()
	self:RegisterEvent("PLAYER_TRADE_MONEY",onUpdate)  -- player money
	self:RegisterEvent("TRADE_MONEY_CHANGED",onUpdate) -- target money
	self:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED",onUpdate) -- player item set
	self:RegisterEvent("TRADE_TARGET_ITEM_CHANGED",onUpdate) -- target item set
	self:RegisterEvent("TRADE_UPDATE",onUpdate)
--	self:RegisterEvent("TRADE_ACCEPT_UPDATE",onUpdateAccept) -- do we need this?
	self:RegisterEvent("TRADE_CLOSED",onClosed)
	self:RegisterEvent("TRADE_REQUEST_CANCEL",onCancel)
end
