--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

-- Disenchanting, Prospecting, Milling
local CB = select(2,...)
local P = CB.Private
local Uncrafting = P:NewModule("Uncrafting", "AceEvent-3.0")
local L = P.L

local DISENCHANT = L["Disenchant"]
local MILLING = L["Milling"]
local PROSPECTING = L["Prospecting"]

local MATCH_SPELL = {}
MATCH_SPELL[L["Disenchant"]]=DISENCHANT -- 3 second cast
MATCH_SPELL[L["Milling"]]=MILLING -- 1 second cast
MATCH_SPELL[L["Prospecting"]]=PROSPECTING -- 2 second cast

local CAST_TIME = {}
CAST_TIME[DISENCHANT]=3
CAST_TIME[MILLING]=1
CAST_TIME[PROSPECTING]=2

local SRC_QUANTITY = {}
SRC_QUANTITY[DISENCHANT]=1
SRC_QUANTITY[MILLING]=5
SRC_QUANTITY[PROSPECTING]=5

local WEAPON_SLOTS={
	["INVTYPE_2HWEAPON"] = true,
	["INVTYPE_WEAPON"] = true,
	["INVTYPE_WEAPONMAINHAND"] = true,
	["INVTYPE_WEAPONOFFHAND"] = true,
	["INVTYPE_RANGED"] = true,
	["INVTYPE_RANGEDRIGHT"] = true,
	["INVTYPE_THROWN"] = true,
}
local ARMOR_SLOTS={
	["INVTYPE_BODY"] = true,
	["INVTYPE_CHEST"] = true,
	["INVTYPE_CLOAK"] = true,
	["INVTYPE_FEET"] = true,
	["INVTYPE_FINGER"] = true,
	["INVTYPE_HAND"] = true,
	["INVTYPE_HEAD"] = true,
	["INVTYPE_HOLDABLE"] = true,
	["INVTYPE_LEGS"] = true,
	["INVTYPE_NECK"] = true,
	["INVTYPE_ROBE"] = true,
	["INVTYPE_SHIELD"] = true,
	["INVTYPE_SHOULDER"] = true,
	["INVTYPE_TABARD"] = true,
	["INVTYPE_TRINKET"] = true,
	["INVTYPE_WAIST"] = true,
	["INVTYPE_WRIST"] = true,
	["INVTYPE_RELIC"] = true,
}
local WEAPON_DE={
	[2]=P.DE_UNCOMMON_WEAPON,
	[3]=P.DE_RARE_WEAPON,
	[4]=P.DE_EPIC_WEAPON,
}
local ARMOR_DE={
	[2]=P.DE_UNCOMMON_ARMOR,
	[3]=P.DE_RARE_ARMOR,
	[4]=P.DE_EPIC_ARMOR,
}

-- these 3 functions are not local so that they may be called by the tooltip
function P.getExpectedDisenchant(item)
	local _, _, quality, iLevel, _, _, _, _, equipSlot, _, _ = GetItemInfo(item)
	if not equipSlot then return end
	if WEAPON_SLOTS[equipSlot] then
		return WEAPON_DE[quality] and WEAPON_DE[quality][iLevel]
	elseif ARMOR_SLOTS[equipSlot] then
		return ARMOR_DE[quality] and ARMOR_DE[quality][iLevel]
	end	
end

function P.getExpectedMilling(item)
	local id = P.idOfItem(item)
	return P.MILL[id]
end

function P.getExpectedProspecting(item)
	local id = P.idOfItem(item)
	return P.PROSPECT[id]
end

-- returns a table mapping item id to count (per cast: 1 de / 5 herbs / 5 ore)
local GET_EXPECTED = {
	[DISENCHANT] = P.getExpectedDisenchant,
	[MILLING] = P.getExpectedMilling,
	[PROSPECTING] = P.getExpectedProspecting,
}

local function onSpellStart(spell)
	local function onItemClick(item,bound)
		local function onSpellSuccess()
			local function onLoot(lootTbl)
--			  P:DebugPrint(lootTbl)
				local exp=GET_EXPECTED[spell](item) or {}
				if bound then
					P:DebugPrint(("Item is %s.  Not recording."):format(bound))
				else
					CB:RecordTransaction(nil,{[P.idOfItem(item)]=SRC_QUANTITY[spell]},lootTbl,exp)
				end
			end
			-- P:ReceiveMultipleCallback(onLoot,P.genericFailCB)
			P:LootCallback(onLoot,P.genericFailCB)
		end
		P:SpellCallback(spell,onSpellSuccess,P.genericFailCB)
	end
	P:ItemCallback(onItemClick,P.genericFailCB)	
end

function Uncrafting:OnInitialize()
	P:AddSpellHandler(L["Disenchant"],onSpellStart)
	P:AddSpellHandler(L["Milling"],onSpellStart)
	P:AddSpellHandler(L["Prospecting"],onSpellStart)
end
